/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <glibmm/i18n.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/widget.h>
#include <gtkmm/main.h>
#include "AircraftDialog.h"
#include "Util.h"

AircraftDialog::AircraftDialog(std::string *GladeFilename, Aircraft *AircraftData)
{
	try
	{
		std::string gladefile;

		refXml = Gnome::Glade::Xml::create(*GladeFilename, "AircraftDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog(_("Error reading data file: ")+*GladeFilename+"\n"+_("Program cannot continue."),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
		return;
	}

	this->GladeFilename=GladeFilename;
	this->AircraftData=AircraftData;

	refXml->get_widget("AircraftDialog",GtkDialog);

	refXml->get_widget("DescEntry",DescEntry);
	refXml->get_widget("RegistrationEntry",RegistrationEntry);
	refXml->get_widget("TASClimbEntry",TASClimbEntry);
	refXml->get_widget("TASCruiseEntry",TASCruiseEntry);
	refXml->get_widget("TASDescentEntry",TASDescentEntry);
	refXml->get_widget("ClimbRateEntry",ClimbRateEntry);
	refXml->get_widget("DescentRateEntry",DescentRateEntry);
	refXml->get_widget("SegmentedRadioButton",SegmentedRadioButton);
	refXml->get_widget("ClimbFuelRateEntry",ClimbFuelRateEntry);
	refXml->get_widget("CruiseFuelRateEntry",CruiseFuelRateEntry);
	refXml->get_widget("DescentFuelRateEntry",DescentFuelRateEntry);
	refXml->get_widget("HourlyRadioButton",HourlyRadioButton);
	refXml->get_widget("Hour1Entry",Hour1Entry);
	refXml->get_widget("Hour2Entry",Hour2Entry);
	refXml->get_widget("Hour3Entry",Hour3Entry);
	refXml->get_widget("Hour4Entry",Hour4Entry);
	refXml->get_widget("Hour5Entry",Hour5Entry);
	refXml->get_widget("Hour6Entry",Hour6Entry);
	refXml->get_widget("Hour7Entry",Hour7Entry);
	refXml->get_widget("Hour8Entry",Hour8Entry);
	refXml->get_widget("Hour9Entry",Hour9Entry);
	refXml->get_widget("AddlHoursEntry",AddlHoursEntry);

	refXml->get_widget("AirspeedUnitsCombo",AirspeedUnitsCombo);
	refXml->get_widget("ClimbDescentRateUnitsCombo",ClimbDescentRateUnitsCombo);
	refXml->get_widget("AltitudeUnitsCombo",AltitudeUnitsCombo);
	refXml->get_widget("AircraftWeightUnitsCombo",AircraftWeightUnitsCombo);
	refXml->get_widget("CGStationDistUnitsCombo",CGStationDistUnitsCombo);
	refXml->get_widget("FuelQtyUnitsCombo",FuelQtyUnitsCombo);
	refXml->get_widget("FuelTypeCombo",FuelTypeCombo);
	refXml->get_widget("OilQtyUnitsCombo",OilQtyUnitsCombo);

	SegmentedRadioButton->signal_clicked().connect(sigc::mem_fun(this,&AircraftDialog::on_SegmentedRadioButton_clicked));
	HourlyRadioButton->signal_clicked().connect(sigc::mem_fun(this,&AircraftDialog::on_HourlyRadioButton_clicked));

	std::string Filename=AircraftData->GetFilename();
	Filename=Filename.substr(Filename.find_last_of("/\\")+1);
	GtkDialog->set_title("Aircraft: "+Filename);

	new AirspeedModelColumns(AirspeedUnitsCombo);
	new ClimbDescentRateModelColumns(ClimbDescentRateUnitsCombo);
	new AltitudeModelColumns(AltitudeUnitsCombo);
	new AircraftWeightModelColumns(AircraftWeightUnitsCombo);
	new CGStationDistModelColumns(CGStationDistUnitsCombo);
	new OilQtyModelColumns(OilQtyUnitsCombo);
	new FuelQtyModelColumns(FuelQtyUnitsCombo);
	new FuelTypeModelColumns(FuelTypeCombo);
}

AircraftDialog::~AircraftDialog() {
	delete GtkDialog;
}

int AircraftDialog::Run() {
	int result;

	SetData();
	SetSensitivities();
	result=GtkDialog->run();
	if (result==Gtk::RESPONSE_OK) {
		GetData();
	}

	return result;
}

void AircraftDialog::SetSensitivities() {
  	if (SegmentedRadioButton->get_active()) {
		ClimbFuelRateEntry->set_sensitive(true);
		CruiseFuelRateEntry->set_sensitive(true);
		DescentFuelRateEntry->set_sensitive(true);
		Hour1Entry->set_sensitive(false);
		Hour2Entry->set_sensitive(false);
		Hour3Entry->set_sensitive(false);
		Hour4Entry->set_sensitive(false);
		Hour5Entry->set_sensitive(false);
		Hour6Entry->set_sensitive(false);
		Hour7Entry->set_sensitive(false);
		Hour8Entry->set_sensitive(false);
		Hour9Entry->set_sensitive(false);
		AddlHoursEntry->set_sensitive(false);
	} else {
		ClimbFuelRateEntry->set_sensitive(false);
		CruiseFuelRateEntry->set_sensitive(false);
		DescentFuelRateEntry->set_sensitive(false);
		Hour1Entry->set_sensitive(true);
		Hour2Entry->set_sensitive(true);
		Hour3Entry->set_sensitive(true);
		Hour4Entry->set_sensitive(true);
		Hour5Entry->set_sensitive(true);
		Hour6Entry->set_sensitive(true);
		Hour7Entry->set_sensitive(true);
		Hour8Entry->set_sensitive(true);
		Hour9Entry->set_sensitive(true);
		AddlHoursEntry->set_sensitive(true);
	}
}

void AircraftDialog::GetData() {
	AircraftData->SetDesc(DescEntry->get_text());
	AircraftData->SetRegistration(RegistrationEntry->get_text());

	AircraftData->SetAirspeedUnits((Aircraft::AirspeedUnitsEnum)AirspeedUnitsCombo->get_active_row_number());
	AircraftData->SetClimbDescentRateUnits((Aircraft::ClimbDescentRateUnitsEnum)ClimbDescentRateUnitsCombo->get_active_row_number());
	AircraftData->SetAltitudeUnits((Aircraft::AltitudeUnitsEnum)AltitudeUnitsCombo->get_active_row_number());
	AircraftData->SetAircraftWeightUnits((Aircraft::AircraftWeightUnitsEnum)AircraftWeightUnitsCombo->get_active_row_number());
	AircraftData->SetCGStationDistUnits((Aircraft::CGStationDistUnitsEnum)CGStationDistUnitsCombo->get_active_row_number());
	AircraftData->SetFuelQtyUnits((Aircraft::FuelQtyUnitsEnum)FuelQtyUnitsCombo->get_active_row_number());
	AircraftData->SetOilQtyUnits((Aircraft::OilQtyUnitsEnum)OilQtyUnitsCombo->get_active_row_number());
	AircraftData->SetFuelType((Aircraft::FuelTypeEnum)FuelTypeCombo->get_active_row_number());

	switch (AircraftData->GetAirspeedUnits()) {
		case Aircraft::AIRSPEEDUNITS_KPH:
			AircraftData->SetTASClimb(atof(TASClimbEntry->get_text().c_str()));
			AircraftData->SetTASCruise(atof(TASCruiseEntry->get_text().c_str()));
			AircraftData->SetTASDescent(atof(TASDescentEntry->get_text().c_str()));
			break;
		case Aircraft::AIRSPEEDUNITS_MPH:
			AircraftData->SetTASClimb(MPH2KPH(atof(TASClimbEntry->get_text().c_str())));
			AircraftData->SetTASCruise(MPH2KPH(atof(TASCruiseEntry->get_text().c_str())));
			AircraftData->SetTASDescent(MPH2KPH(atof(TASDescentEntry->get_text().c_str())));
			break;
		case Aircraft::AIRSPEEDUNITS_KNOTS:
			AircraftData->SetTASClimb(KT2KPH(atof(TASClimbEntry->get_text().c_str())));
			AircraftData->SetTASCruise(KT2KPH(atof(TASCruiseEntry->get_text().c_str())));
			AircraftData->SetTASDescent(KT2KPH(atof(TASDescentEntry->get_text().c_str())));
			break;
		case Aircraft::AIRSPEEDUNITS_INVALID:
			/* Do nothing */
			break;
	}

	switch (AircraftData->GetClimbDescentRateUnits()) {
		case Aircraft::CLIMBDESCENTRATEUNITS_MPERS:
			AircraftData->SetClimbRate(atof(ClimbRateEntry->get_text().c_str()));
			AircraftData->SetDescentRate(atof(DescentRateEntry->get_text().c_str()));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_FTPERMIN:
			AircraftData->SetClimbRate(FTPERMIN2MPERS(atof(ClimbRateEntry->get_text().c_str())));
			AircraftData->SetDescentRate(FTPERMIN2MPERS(atof(DescentRateEntry->get_text().c_str())));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_KNOTS:
			AircraftData->SetClimbRate(KT2MPERS(atof(ClimbRateEntry->get_text().c_str())));
			AircraftData->SetDescentRate(KT2MPERS(atof(DescentRateEntry->get_text().c_str())));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_INVALID:
			/* Do nothing */
			break;
	}

	if (SegmentedRadioButton->get_active()) {
		AircraftData->SetFuelRateModel(Aircraft::FUELRATEMODEL_SEGMENT);

		switch (AircraftData->GetFuelQtyUnits()) {
			case Aircraft::FUELQTYUNITS_KG:
				AircraftData->SetFuelRateClimb(atof(ClimbFuelRateEntry->get_text().c_str()));
				AircraftData->SetFuelRateCruise(atof(CruiseFuelRateEntry->get_text().c_str()));
				AircraftData->SetFuelRateDescent(atof(DescentFuelRateEntry->get_text().c_str()));
				break;
			case Aircraft::FUELQTYUNITS_L:
				AircraftData->SetFuelRateClimb(atof(ClimbFuelRateEntry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				AircraftData->SetFuelRateCruise(atof(CruiseFuelRateEntry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				AircraftData->SetFuelRateDescent(atof(DescentFuelRateEntry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				break;
			case Aircraft::FUELQTYUNITS_LB:
				AircraftData->SetFuelRateClimb(LB2KG(atof(ClimbFuelRateEntry->get_text().c_str())));
				AircraftData->SetFuelRateCruise(LB2KG(atof(CruiseFuelRateEntry->get_text().c_str())));
				AircraftData->SetFuelRateDescent(LB2KG(atof(DescentFuelRateEntry->get_text().c_str())));
				break;
			case Aircraft::FUELQTYUNITS_GAL:
				AircraftData->SetFuelRateClimb(GAL2L(atof(ClimbFuelRateEntry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				AircraftData->SetFuelRateCruise(GAL2L(atof(CruiseFuelRateEntry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				AircraftData->SetFuelRateDescent(GAL2L(atof(DescentFuelRateEntry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]);
				break;
			case Aircraft::FUELQTYUNITS_INVALID:
				/* Do nothing */
				break;
		}
	} else if (HourlyRadioButton->get_active()) {
		AircraftData->SetFuelRateModel(Aircraft::FUELRATEMODEL_HOURLY);

		switch (AircraftData->GetFuelQtyUnits()) {
			case Aircraft::FUELQTYUNITS_KG:
				AircraftData->SetFuelRateHourly(atof(Hour1Entry->get_text().c_str()),0);
				AircraftData->SetFuelRateHourly(atof(Hour2Entry->get_text().c_str()),1);
				AircraftData->SetFuelRateHourly(atof(Hour3Entry->get_text().c_str()),2);
				AircraftData->SetFuelRateHourly(atof(Hour4Entry->get_text().c_str()),3);
				AircraftData->SetFuelRateHourly(atof(Hour5Entry->get_text().c_str()),4);
				AircraftData->SetFuelRateHourly(atof(Hour6Entry->get_text().c_str()),5);
				AircraftData->SetFuelRateHourly(atof(Hour7Entry->get_text().c_str()),6);
				AircraftData->SetFuelRateHourly(atof(Hour8Entry->get_text().c_str()),7);
				AircraftData->SetFuelRateHourly(atof(Hour9Entry->get_text().c_str()),8);
				AircraftData->SetFuelRateHourly(atof(AddlHoursEntry->get_text().c_str()),9);
				break;
			case Aircraft::FUELQTYUNITS_L:
				AircraftData->SetFuelRateHourly(atof(Hour1Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],0);
				AircraftData->SetFuelRateHourly(atof(Hour2Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],1);
				AircraftData->SetFuelRateHourly(atof(Hour3Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],2);
				AircraftData->SetFuelRateHourly(atof(Hour4Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],3);
				AircraftData->SetFuelRateHourly(atof(Hour5Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],4);
				AircraftData->SetFuelRateHourly(atof(Hour6Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],5);
				AircraftData->SetFuelRateHourly(atof(Hour7Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],6);
				AircraftData->SetFuelRateHourly(atof(Hour8Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],7);
				AircraftData->SetFuelRateHourly(atof(Hour9Entry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],8);
				AircraftData->SetFuelRateHourly(atof(AddlHoursEntry->get_text().c_str())*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],9);
			case Aircraft::FUELQTYUNITS_LB:
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour1Entry->get_text().c_str())),0);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour2Entry->get_text().c_str())),1);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour3Entry->get_text().c_str())),2);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour4Entry->get_text().c_str())),3);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour5Entry->get_text().c_str())),4);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour6Entry->get_text().c_str())),5);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour7Entry->get_text().c_str())),6);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour8Entry->get_text().c_str())),7);
				AircraftData->SetFuelRateHourly(LB2KG(atof(Hour9Entry->get_text().c_str())),8);
				AircraftData->SetFuelRateHourly(LB2KG(atof(AddlHoursEntry->get_text().c_str())),9);
			case Aircraft::FUELQTYUNITS_GAL:
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour1Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],0);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour2Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],1);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour3Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],2);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour4Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],3);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour5Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],4);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour6Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],5);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour7Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],6);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour8Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],7);
				AircraftData->SetFuelRateHourly(GAL2L(atof(Hour9Entry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],8);
				AircraftData->SetFuelRateHourly(GAL2L(atof(AddlHoursEntry->get_text().c_str()))*Aircraft::FuelTypeDensity[AircraftData->GetFuelType()],9);
			case Aircraft::FUELQTYUNITS_INVALID:
				/* Do nothing */
			break;
		}
	} else {
		AircraftData->SetFuelRateModel(Aircraft::FUELRATEMODEL_INVALID);
	}
}

void AircraftDialog::SetData() {
	DescEntry->set_text(AircraftData->GetDesc());
	RegistrationEntry->set_text(AircraftData->GetRegistration());

	switch (AircraftData->GetAirspeedUnits()) {
		case Aircraft::AIRSPEEDUNITS_KPH:
			TASClimbEntry->set_text(Util::DoubleToStr(AircraftData->GetTASClimb()));
			TASCruiseEntry->set_text(Util::DoubleToStr(AircraftData->GetTASCruise()));
			TASDescentEntry->set_text(Util::DoubleToStr(AircraftData->GetTASDescent()));
			break;
		case Aircraft::AIRSPEEDUNITS_MPH:
			TASClimbEntry->set_text(Util::DoubleToStr(KPH2MPH(AircraftData->GetTASClimb())));
			TASCruiseEntry->set_text(Util::DoubleToStr(KPH2MPH(AircraftData->GetTASCruise())));
			TASDescentEntry->set_text(Util::DoubleToStr(KPH2MPH(AircraftData->GetTASDescent())));
			break;
		case Aircraft::AIRSPEEDUNITS_KNOTS:
			TASClimbEntry->set_text(Util::DoubleToStr(KPH2KT(AircraftData->GetTASClimb())));
			TASCruiseEntry->set_text(Util::DoubleToStr(KPH2KT(AircraftData->GetTASCruise())));
			TASDescentEntry->set_text(Util::DoubleToStr(KPH2KT(AircraftData->GetTASDescent())));
			break;
		case Aircraft::AIRSPEEDUNITS_INVALID:
			/* Do nothing */
			break;
	}

	switch (AircraftData->GetClimbDescentRateUnits()) {
		case Aircraft::CLIMBDESCENTRATEUNITS_MPERS:
			ClimbRateEntry->set_text(Util::DoubleToStr(AircraftData->GetClimbRate()));
			DescentRateEntry->set_text(Util::DoubleToStr(AircraftData->GetDescentRate()));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_FTPERMIN:
			ClimbRateEntry->set_text(Util::DoubleToStr(MPERS2FTPERMIN(AircraftData->GetClimbRate())));
			DescentRateEntry->set_text(Util::DoubleToStr(MPERS2FTPERMIN(AircraftData->GetDescentRate())));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_KNOTS:
			ClimbRateEntry->set_text(Util::DoubleToStr(MPERS2KT(AircraftData->GetClimbRate())));
			DescentRateEntry->set_text(Util::DoubleToStr(MPERS2KT(AircraftData->GetDescentRate())));
			break;
		case Aircraft::CLIMBDESCENTRATEUNITS_INVALID:
			/* Do nothing */
			break;
	}

	SegmentedRadioButton->set_active(AircraftData->GetFuelRateModel()==Aircraft::FUELRATEMODEL_SEGMENT);
	switch (AircraftData->GetFuelQtyUnits()) {
		case Aircraft::FUELQTYUNITS_KG:
			ClimbFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateClimb()));
			CruiseFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateCruise()));
			DescentFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateDescent()));

			Hour1Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(0)));
			Hour2Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(1)));
			Hour3Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(2)));
			Hour4Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(3)));
			Hour5Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(4)));
			Hour6Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(5)));
			Hour7Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(6)));
			Hour8Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(7)));
			Hour9Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(8)));
			AddlHoursEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(9)));
			break;
		case Aircraft::FUELQTYUNITS_L:
			ClimbFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateClimb()/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			CruiseFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateCruise()/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			DescentFuelRateEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateDescent()/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));

			Hour1Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(0)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour2Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(1)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour3Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(2)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour4Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(3)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour5Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(4)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour6Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(5)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour7Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(6)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour8Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(7)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour9Entry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(8)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			AddlHoursEntry->set_text(Util::DoubleToStr(AircraftData->GetFuelRateHourly(9)/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			break;
		case Aircraft::FUELQTYUNITS_LB:
			ClimbFuelRateEntry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateClimb())));
			CruiseFuelRateEntry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateCruise())));
			DescentFuelRateEntry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateDescent())));

			Hour1Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(0))));
			Hour2Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(1))));
			Hour3Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(2))));
			Hour4Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(3))));
			Hour5Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(4))));
			Hour6Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(5))));
			Hour7Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(6))));
			Hour8Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(7))));
			Hour9Entry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(8))));
			AddlHoursEntry->set_text(Util::DoubleToStr(KG2LB(AircraftData->GetFuelRateHourly(9))));
			break;
		case Aircraft::FUELQTYUNITS_GAL:
			ClimbFuelRateEntry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateClimb())/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			CruiseFuelRateEntry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateCruise())/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			DescentFuelRateEntry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateDescent())/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));

			Hour1Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(0))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour2Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(1))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour3Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(2))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour4Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(3))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour5Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(4))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour6Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(5))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour7Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(6))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour8Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(7))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			Hour9Entry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(8))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			AddlHoursEntry->set_text(Util::DoubleToStr(L2GAL(AircraftData->GetFuelRateHourly(9))/Aircraft::FuelTypeDensity[AircraftData->GetFuelType()]));
			break;
		case Aircraft::FUELQTYUNITS_INVALID:
			/* Do nothing */
			break;
	}

	HourlyRadioButton->set_active(AircraftData->GetFuelRateModel()==Aircraft::FUELRATEMODEL_HOURLY);

	AirspeedUnitsCombo->set_active((int)AircraftData->GetAirspeedUnits());
	ClimbDescentRateUnitsCombo->set_active((int)AircraftData->GetClimbDescentRateUnits());
	AltitudeUnitsCombo->set_active((int)AircraftData->GetAltitudeUnits());
	AircraftWeightUnitsCombo->set_active((int)AircraftData->GetAircraftWeightUnits());
	CGStationDistUnitsCombo->set_active((int)AircraftData->GetCGStationDistUnits());
	FuelQtyUnitsCombo->set_active((int)AircraftData->GetFuelQtyUnits());
	OilQtyUnitsCombo->set_active((int)AircraftData->GetOilQtyUnits());
	FuelTypeCombo->set_active((int)AircraftData->GetFuelType());
}

void AircraftDialog::on_SegmentedRadioButton_clicked() {
	SetSensitivities();
}

void AircraftDialog::on_HourlyRadioButton_clicked() {
	SetSensitivities();
}

AircraftDialog::AirspeedModelColumns::AirspeedModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Airspeed Units ComboBox */
	AirspeedrefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(AirspeedrefTreeModel);

	for (int i=0;i<Aircraft::AIRSPEEDUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(AirspeedrefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::AirspeedUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::ClimbDescentRateModelColumns::ClimbDescentRateModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Climb Rate Units ComboBox */
	ClimbDescentRaterefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(ClimbDescentRaterefTreeModel);

	for (int i=0;i<Aircraft::CLIMBDESCENTRATEUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(ClimbDescentRaterefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::ClimbDescentRateUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::AltitudeModelColumns::AltitudeModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Altitude Units ComboBox */
	AltituderefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(AltituderefTreeModel);

	for (int i=0;i<Aircraft::ALTITUDEUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(AltituderefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::AltitudeUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::AircraftWeightModelColumns::AircraftWeightModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Aircraft Weight Units ComboBox */
	AircraftWeightrefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(AircraftWeightrefTreeModel);

	for (int i=0;i<Aircraft::AIRCRAFTWEIGHTUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(AircraftWeightrefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::AircraftWeightUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::CGStationDistModelColumns::CGStationDistModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the CG Station Dist Units ComboBox */
	CGStationDistrefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(CGStationDistrefTreeModel);

	for (int i=0;i<Aircraft::CGSTATIONDISTUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(CGStationDistrefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::CGStationDistUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::OilQtyModelColumns::OilQtyModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the OilQtyUnits ComboBox */
	OilQtyrefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(OilQtyrefTreeModel);

	for (int i=0;i<Aircraft::OILQTYUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(OilQtyrefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::OilQtyUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::FuelQtyModelColumns::FuelQtyModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Fuel Qty Units ComboBox */
	FuelQtyrefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(FuelQtyrefTreeModel);

	for (int i=0;i<Aircraft::FUELQTYUNITS_INVALID;i++) {
		Gtk::TreeModel::Row row = *(FuelQtyrefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::FuelQtyUnitsText[i];
	}

	Combo->pack_start(Name);
}

AircraftDialog::FuelTypeModelColumns::FuelTypeModelColumns(Gtk::ComboBox *Combo) {
	add(Id);
	add(Name);

	/* Setup the Fuel Type Units ComboBox */
	FuelTyperefTreeModel = Gtk::ListStore::create(*this);
	Combo->set_model(FuelTyperefTreeModel);

	for (int i=0;i<Aircraft::FUELTYPE_INVALID;i++) {
		Gtk::TreeModel::Row row = *(FuelTyperefTreeModel->append());
		row[Id] = i;
		row[Name] = Aircraft::FuelTypeText[i];
	}

	Combo->pack_start(Name);
}
