/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef UTIL_H_
#define UTIL_H_

#ifdef HAVE_CONFIG_H
 #include "../config.h"
#endif

#include <string>
#include <vector>
#ifdef HAVE_ENDIAN_H
 #include <endian.h>
#endif

#include "OverwriteFileDialog.h"

/* Various unit conversions */

#define DEG2RAD(d) ((d)*(M_PI/180.0))
#define RAD2DEG(r) ((r)/(M_PI/180.0))

#define METERS2NM(m) ((m)*(1.0/1852))
#define NM2METERS(nm) ((nm)*1852)

#define METERS2FT(m) ((m)*3.2808399)
#define FT2METERS(ft) ((ft)/3.2808399)

#define MPERS2KT(mpers) ((mpers)*1.9438445)
#define KT2MPERS(kt) ((kt)/1.9438445)

#define KT2KPH(kt) ((kt)*1.85200)
#define KPH2KT(kph) ((kph)/1.85200)

#define MPH2KPH(mph) ((mph)*1.609344)
#define KPH2MPH(kph) ((kph)/1.609344)

#define FTPERMIN2KPH(ftm) ((ftm)/54.6806649)
#define KPH2FTPERMIN(kmh) ((kph)*54.6806649)

#define FTPERMIN2MPERS(fperm) ((fperm)*0.00508)
#define MPERS2FTPERMIN(mpers) ((mpers)/0.00508)

#define LB2KG(lb) ((lb)/2.20462262)
#define KG2LB(kg) ((kg)*2.20462262)

#define MILES2KM(mi) ((mi)*1.609344)
#define KM2MILES(km) ((km)/1.609344)

#define GAL2L(gal) ((gal)*3.78541178)
#define L2GAL(l) ((l)/3.78541178)

#define QT2L(qt) ((qt)*0.946352946)
#define L2QT(l) ((l)/0.946352946)

#define LAVGAS2KG(l) ((l)*0.721355092)
#define KG2LAVGAS(kg) ((kg)/0.721355092)

#define LMOGAS2KG(l) ((l)*0.727705893)
#define KG2LMOGAS(kg) ((kg)/0.727705893)

#define LJET2KG(l) ((l)*0.8075)
#define KG2LJET(kg) ((kg)/0.8075)

#define LOIL2KG(l) ((l)*0.874732919)
#define KG2LOIL(kg) ((kg)/0.874732919)

/* If endian conversions aren't already defined, assume we're on a
 * little-endian machine and define them ourselves.
 */
#ifndef le32toh
 #define le32toh(x) (x)
#endif

#ifndef htole32
 #define htole32(x) (x)
#endif

#ifndef be32toh
 #define be32toh(x) ( (x&0xFF000000)>>24 | (x&0x00FF0000)>>8 | (x&0x0000FF00)<<8 | (x&0x000000FF)<<24 )
#endif

#ifndef htobe32
 #define htobe32(x) ( (x&0xFF000000)>>24 | (x&0x00FF0000)>>8 | (x&0x0000FF00)<<8 | (x&0x000000FF)<<24 )
#endif

/* For Win32 portability */
#ifndef WIN32
 #define UTIL_isnan isnan
#else
 #define UTIL_isnan _isnan
#endif

class Util
{
public:
	static void ShellExpand(std::string *str);
	static void ChkDir(std::string DirPath);
	static time_t GMMkTime(struct tm *tm);
	static void GmTime_r(struct tm *bdtime, time_t Timestamp);
	static bool SmartCopyFile(std::string SrcPath, std::string DestDir, std::string DestName="", OverwriteFileDialog *OFD=NULL);
	static void ScheduleMainLoop();
	static std::string toUpper(std::string str);
	static std::string DoubleToStr(const double Number);
	static void Explode(const std::string Str, const std::string Delim, std::vector<std::string> *Tokens);
	static void LonLatToDMMSSPSSSS(double Lon, double Lat, std::string *str);
	static void LonLatToDMMPMMMMMM(double Lon, double Lat, std::string *str);
	static void LonLatToDPDDDDDDD(double Lon, double Lat, std::string *str);
	static std::string BDTimeToMM_DD_HH_MM(struct tm *bdtime);
	static time_t CalcEffectiveFrom(std::string *Volume);
	static time_t CalcEffectiveTo(std::string *Volume);

	/* Used for Win32 portability */
	static int MkDir(std::string Path, int Mode=0755);
	static char *StrError();
};

#endif /*UTIL_H_*/
